/********************************************************************
 * BenchIT - Performance Measurement for Scientific Applications
 * Contact: developer@benchit.org
 *
 * $Id: BIEnvHash.template.java 1 2009-09-11 12:26:19Z william $
 * $URL: svn+ssh://benchit@rupert.zih.tu-dresden.de/svn-base/benchit-root/BenchITv6/tools/BIEnvHash.template.java $
 * For license details see COPYING in the package base directory
 *******************************************************************/
import java.util.HashMap;
import java.util.Iterator;
import java.io.*;

public class BIEnvHash
{
   private static BIEnvHash instance = null;
   private HashMap table = null;
   private BIEnvHash()
   {
      bi_initTable();
      bi_fillTable();
   }
   public static BIEnvHash getInstance()
   {
      if ( instance == null ) instance = new BIEnvHash();
      return instance;
   }
   /** Puts a new key value mapping into the map and returns the
     * previously assigned value to the key, or null, if it's a
     * new mapping. */
   public String bi_setEnv( String key, String value )
   {
      return bi_put( key, value );
   }
   /** Puts a new key value mapping into the map and returns the
     * previously assigned value to the key, or null, if it's a
     * new mapping. */
   public String bi_put( String key, String value )
   {
      if ( table == null ) bi_initTable();
      Object ret = table.put( key, value );
      if ( ret == null ) return null;
      return (String)ret;
   }
   public String bi_getEnv( String key )
   {
      if (System.getenv(key)!=null) return System.getenv(key);
      return bi_get( key );
   }
   public String bi_get( String key )
   {
      if ( table == null ) return null;
      return (String)(table.get( key ));
   }
   public void bi_dumpTable()
   {
      if ( table == null ) return;
      printf( "\nHashtable dump of all known environment variables at compiletime:" );
      printf( "\n Key            | Value" );
      printf( "\n----------------------------------" );
      Iterator it = table.keySet().iterator();
      while ( it.hasNext() )
      {
         String key = (String)(it.next());
         String value = (String)(table.get( key ));
         printf( "\n " + key + " | " + value );
      }
      printf( "\n" + bi_size() + " entries in total.\n" );
      flush();
   }
   public void dumpToOutputBuffer( StringBuffer outputBuffer )
   {
      outputBuffer.append( "beginofenvironmentvariables\n" );
      Iterator it = table.keySet().iterator();
      while ( it.hasNext() )
      {
         String key = (String)(it.next());
         String value = (String)(table.get( key ));
         StringBuffer b = new StringBuffer();
         for ( int i = 0; i < value.length(); i++ )
         {
            if ( value.charAt( i ) == '"' ) {
               /* check equal number of escape chars->insert escape char */
               int cnt = 0;
               for ( int j = i - 1;  j >= 0; j-- ) {
                  if ( value.charAt( j ) == '\\' ) cnt++;
               }
               /* escape quote */
               if ( cnt % 2 == 0 ) {
                  b.append( '\\' );
               }
            }
            b.append( value.charAt( i ) );
         }
         value = b.toString();
         outputBuffer.append( key + "=\"" + value + "\"\n" );
      }
      outputBuffer.append( "endofenvironmentvariables\n" );
   }
   public int bi_size()
   {
      if ( table != null ) return table.size();
      return -1;
   }
   public void bi_initTable()
   {
      table = new HashMap( 1009 );
   }
   private void printf( String str )
   {
      System.out.print( str );
   }
   private void flush()
   {
      System.out.flush();
      System.err.flush();
   }
   /** Takes a PARAMETER file as argument and adds it's variables
     * to the environment HashMap. */
   public boolean bi_readParameterFile( String fileName )
   {
      boolean retval = false;
      File textFile = null;
      try {
         textFile = new File( fileName );
      }
      catch ( NullPointerException npe ) {
         return retval;
      }
      if ( textFile != null ) {
         if ( textFile.exists() ) {
            FileReader in;
            char ch = '\0';
            int size = (int)(textFile.length());
            int read = 0;
            char inData[] = new char[size + 1];
            try {
               in = new FileReader( textFile );
               in.read( inData, 0, size );
               in.close();
            }
            catch( IOException ex ) {
               return retval;
            } // scan inData for environment variables
            int scanned = 0;
            int it = 0;
            ch = inData[scanned];
            StringBuffer buffer = null;
            while ( scanned < size ) {
               buffer = new StringBuffer();
               // read a line
               while ( ch != '\n' ) {
                  if ( ch != '\r' ) {
                     buffer.append( ch );
                  }
                  scanned += 1;
                  ch = inData[scanned];
               }
               // trim leading and trailing white spaces
               String lineValue = buffer.toString().trim();
               // now check the line
               int eqIndex = -1;
               try {
                  eqIndex = lineValue.indexOf( "=" );
               }
               catch ( NullPointerException npe ) {
                  return retval;
               }
               // only check lines containing an equals sign, no $ sign,
               // and the start with a capital letter
               if ( eqIndex >= 0 && lineValue.indexOf( "$" ) < 0 &&
                  lineValue.substring( 0, 1 ).matches( "[A-Z]" ) ) {
                  String varname = null;
                  String varvalue = null;
                  try {
                     varname = lineValue.substring( 0 , eqIndex );
                     varvalue = lineValue.substring( eqIndex + 1 );
                  }
                  catch ( IndexOutOfBoundsException ioobe ) {
                  }
                  if ( ( varname != null ) && ( varvalue != null ) ) {
                     boolean st1=false, en1=false, st2=false, en2=false;
                     boolean st3=false, en3=false;
                     try {
                        st1=varvalue.startsWith( "'" );
                        en1=varvalue.endsWith( "'" );
                        st2=varvalue.startsWith( "(" );
                        en2=varvalue.endsWith( ")" );
                        st3=varvalue.startsWith( "\"" );
                        en3=varvalue.endsWith( "\"" );
                     }
                     catch ( NullPointerException npe ) {
                        return retval;
                     }
                     if ( ( ( st1 == en1 ) && ( ( st1 != st2 ) && ( st1!=st3 ) ) ) ||
                          ( ( st2 == en2 ) && ( ( st2 != st1 ) && ( st2!=st3 ) ) ) ||
                          ( ( st3 == en3 ) && ( ( st3 != st1 ) && ( st3!=st2 ) ) ) ||
                          ( ( st1 == en1 ) && ( st2 == en2 ) && ( st3 == en3 )
                          && ( st1 == st2 ) && ( st2 == st3 ) && ( st3 == false ) ) ) {
                        table.put( varname, varvalue );
                     }
                  }
               }
               scanned += 1;
               ch = inData[scanned];
               it++;
            }
         }
         else {
            System.err.println( "BenchIT: PARAMETER file \"" + fileName
               + "\" doesn't exist." );
            return retval;
         }
      }
      else {
         System.err.println( "BenchIT: PARAMETER file \"" + fileName
               + "\" not found." );
         return retval;
      }
      return true;
   }
   /* special case: generated from outside and code will be
      appended to this file */
   /** Fills the table with predefined content. */
   public void bi_fillTable()
   {

      bi_put( "BENCHITROOT", "/home/herm/BenchITv6" );
      bi_put( "BENCHIT_ARCH_SHORT", "IVY" );
      bi_put( "BENCHIT_ARCH_SPEED", "unknownM" );
      bi_put( "BENCHIT_CC", "gcc" );
      bi_put( "BENCHIT_CC_C_FLAGS", "" );
      bi_put( "BENCHIT_CC_C_FLAGS_HIGH", "-O0" );
      bi_put( "BENCHIT_CC_C_FLAGS_OMP", "" );
      bi_put( "BENCHIT_CC_C_FLAGS_STD", "-O2" );
      bi_put( "BENCHIT_CC_LD", "gcc" );
      bi_put( "BENCHIT_CC_L_FLAGS", "-lm" );
      bi_put( "BENCHIT_CPP_ACML", "" );
      bi_put( "BENCHIT_CPP_ATLAS", "" );
      bi_put( "BENCHIT_CPP_BLAS", "" );
      bi_put( "BENCHIT_CPP_ESSL", "" );
      bi_put( "BENCHIT_CPP_FFTW3", "" );
      bi_put( "BENCHIT_CPP_MKL", "" );
      bi_put( "BENCHIT_CPP_MPI", " -DUSE_MPI" );
      bi_put( "BENCHIT_CPP_PAPI", "-DUSE_PAPI" );
      bi_put( "BENCHIT_CPP_PCL", " -DUSE_PCL" );
      bi_put( "BENCHIT_CPP_PTHREADS", "" );
      bi_put( "BENCHIT_CPP_PVM", "" );
      bi_put( "BENCHIT_CPP_SCSL", "" );
      bi_put( "BENCHIT_CROSSCOMPILE", "0" );
      bi_put( "BENCHIT_CXX", "g++" );
      bi_put( "BENCHIT_CXX_C_FLAGS", "" );
      bi_put( "BENCHIT_CXX_C_FLAGS_HIGH", "-O3" );
      bi_put( "BENCHIT_CXX_C_FLAGS_OMP", "" );
      bi_put( "BENCHIT_CXX_C_FLAGS_STD", "-O2" );
      bi_put( "BENCHIT_CXX_LD", "g++" );
      bi_put( "BENCHIT_CXX_L_FLAGS", "-lm" );
      bi_put( "BENCHIT_DEBUGLEVEL", "0" );
      bi_put( "BENCHIT_DEFINES", " -DDEBUGLEVEL=0" );
      bi_put( "BENCHIT_ENVIRONMENT", "DEFAULT" );
      bi_put( "BENCHIT_F77", "gfortran" );
      bi_put( "BENCHIT_F77_C_FLAGS", "" );
      bi_put( "BENCHIT_F77_C_FLAGS_HIGH", "-O3" );
      bi_put( "BENCHIT_F77_C_FLAGS_OMP", "" );
      bi_put( "BENCHIT_F77_C_FLAGS_STD", "-O2" );
      bi_put( "BENCHIT_F77_LD", "gfortran" );
      bi_put( "BENCHIT_F77_L_FLAGS", "-lm" );
      bi_put( "BENCHIT_F90", "gfortran" );
      bi_put( "BENCHIT_F90_C_FLAGS", "" );
      bi_put( "BENCHIT_F90_C_FLAGS_HIGH", "-O3" );
      bi_put( "BENCHIT_F90_C_FLAGS_OMP", "" );
      bi_put( "BENCHIT_F90_C_FLAGS_STD", "-O2" );
      bi_put( "BENCHIT_F90_LD", "gfortran" );
      bi_put( "BENCHIT_F90_L_FLAGS", "-lm" );
      bi_put( "BENCHIT_F90_SOURCE_FORMAT_FLAG", "" );
      bi_put( "BENCHIT_F95", "gfortran" );
      bi_put( "BENCHIT_F95_C_FLAGS", "" );
      bi_put( "BENCHIT_F95_C_FLAGS_HIGH", "-O4" );
      bi_put( "BENCHIT_F95_C_FLAGS_OMP", "" );
      bi_put( "BENCHIT_F95_C_FLAGS_STD", "-O2" );
      bi_put( "BENCHIT_F95_LD", "gfortran" );
      bi_put( "BENCHIT_F95_L_FLAGS", "-lm" );
      bi_put( "BENCHIT_F95_SOURCE_FORMAT_FLAG", "" );
      bi_put( "BENCHIT_FILENAME_COMMENT", "0" );
      bi_put( "BENCHIT_HOSTNAME", "herm-ThinkPad-X220" );
      bi_put( "BENCHIT_IGNORE_PARAMETER_FILE", "0" );
      bi_put( "BENCHIT_INCLUDES", "-I. -I/home/herm/BenchITv6 -I/home/herm/BenchITv6/tools" );
      bi_put( "BENCHIT_INTERACTIVE", "0" );
      bi_put( "BENCHIT_JAVA", "java" );
      bi_put( "BENCHIT_JAVAC", "javac" );
      bi_put( "BENCHIT_JAVAC_FLAGS", "" );
      bi_put( "BENCHIT_JAVAC_FLAGS_HIGH", "-O" );
      bi_put( "BENCHIT_JAVA_FLAGS", "" );
      bi_put( "BENCHIT_JAVA_HOME", "" );
      bi_put( "BENCHIT_KERNELBINARY", "/home/herm/BenchITv6/bin/arch_x86_64.memory_bandwidth.C.pthread.SSE2-Flush.single-cas-32-fail.0" );
      bi_put( "BENCHIT_KERNELNAME", "arch_x86_64.memory_bandwidth.C.pthread.SSE2-Flush.single-cas-32-fail" );
      bi_put( "BENCHIT_KERNEL_ALLOC", "L" );
      bi_put( "BENCHIT_KERNEL_BURST_LENGTH", "4" );
      bi_put( "BENCHIT_KERNEL_COMMENT", "memory write bandwidth" );
      bi_put( "BENCHIT_KERNEL_COUNTERS", "PAPI_L2_TCM" );
      bi_put( "BENCHIT_KERNEL_CPU_LIST", "0,2,3" );
      bi_put( "BENCHIT_KERNEL_ENABLE_PAPI", "0" );
      bi_put( "BENCHIT_KERNEL_FLUSH_ACCESSES", "2" );
      bi_put( "BENCHIT_KERNEL_FLUSH_L1", "1" );
      bi_put( "BENCHIT_KERNEL_FLUSH_L2", "1" );
      bi_put( "BENCHIT_KERNEL_FLUSH_L3", "1" );
      bi_put( "BENCHIT_KERNEL_FLUSH_MODE", "M" );
      bi_put( "BENCHIT_KERNEL_HUGEPAGES", "1" );
      bi_put( "BENCHIT_KERNEL_HUGEPAGE_DIR", "/mnt/huge" );
      bi_put( "BENCHIT_KERNEL_INSTRUCTION", "mov" );
      bi_put( "BENCHIT_KERNEL_MAX", "200000000" );
      bi_put( "BENCHIT_KERNEL_MIN", "4000" );
      bi_put( "BENCHIT_KERNEL_OFFSET", "0" );
      bi_put( "BENCHIT_KERNEL_RUNS", "4" );
      bi_put( "BENCHIT_KERNEL_SERIALIZATION", "mfence" );
      bi_put( "BENCHIT_KERNEL_SHARE_CPU", "3" );
      bi_put( "BENCHIT_KERNEL_STEPS", "100" );
      bi_put( "BENCHIT_KERNEL_TIMEOUT", "3600" );
      bi_put( "BENCHIT_KERNEL_USE_ACCESSES", "1" );
      bi_put( "BENCHIT_KERNEL_USE_DIRECTION", "FIFO" );
      bi_put( "BENCHIT_KERNEL_USE_MODE", "M" );
      bi_put( "BENCHIT_LD_LIBRARY_PATH", "/home/herm/BenchITv6/jbi/jni" );
      bi_put( "BENCHIT_LIB_ACML", " -lacml" );
      bi_put( "BENCHIT_LIB_ATLAS", " -latlas" );
      bi_put( "BENCHIT_LIB_BLAS", "-lblas" );
      bi_put( "BENCHIT_LIB_ESSL", " -lessl" );
      bi_put( "BENCHIT_LIB_FFTW3", " -lfftw3" );
      bi_put( "BENCHIT_LIB_MKL", " -lmkl" );
      bi_put( "BENCHIT_LIB_MPI", "" );
      bi_put( "BENCHIT_LIB_PAPI", "" );
      bi_put( "BENCHIT_LIB_PCL", "" );
      bi_put( "BENCHIT_LIB_PTHREAD", "" );
      bi_put( "BENCHIT_LIB_PVM", "" );
      bi_put( "BENCHIT_LIB_SCSL", " -lscsl" );
      bi_put( "BENCHIT_LOCAL_CC", "gcc" );
      bi_put( "BENCHIT_LOCAL_CC_C_FLAGS", "" );
      bi_put( "BENCHIT_LOCAL_CC_L_FLAGS", "-lm" );
      bi_put( "BENCHIT_MANDATORY_FILES", "benchit.c interface.h tools/envhashbuilder.c tools/bienvhash.template.c tools/bienvhash.h tools/stringlib.c tools/stringlib.h " );
      bi_put( "BENCHIT_MPICC", "gcc" );
      bi_put( "BENCHIT_MPICC_C_FLAGS", "" );
      bi_put( "BENCHIT_MPICC_C_FLAGS_HIGH", "-O3" );
      bi_put( "BENCHIT_MPICC_C_FLAGS_OMP", "" );
      bi_put( "BENCHIT_MPICC_C_FLAGS_STD", "-O2" );
      bi_put( "BENCHIT_MPICC_LD", "gcc" );
      bi_put( "BENCHIT_MPICC_L_FLAGS", "-lm -lmpi" );
      bi_put( "BENCHIT_MPIF77", "" );
      bi_put( "BENCHIT_MPIF77_C_FLAGS", "" );
      bi_put( "BENCHIT_MPIF77_C_FLAGS_HIGH", "" );
      bi_put( "BENCHIT_MPIF77_C_FLAGS_OMP", "" );
      bi_put( "BENCHIT_MPIF77_C_FLAGS_STD", "" );
      bi_put( "BENCHIT_MPIF77_LD", "" );
      bi_put( "BENCHIT_MPIF77_L_FLAGS", "" );
      bi_put( "BENCHIT_MPIRUN", "mpirun" );
      bi_put( "BENCHIT_NODENAME", "herm-ThinkPad-X220" );
      bi_put( "BENCHIT_NUM_CPUS", "2" );
      bi_put( "BENCHIT_NUM_PROCESSES", "" );
      bi_put( "BENCHIT_NUM_THREADS_PER_PROCESS", "" );
      bi_put( "BENCHIT_OPTIONAL_FILES", "LOCALDEFS/PROTOTYPE_input_architecture LOCALDEFS/PROTOTYPE_input_display " );
      bi_put( "BENCHIT_PARAMETER_FILE", "/home/herm/BenchITv6/kernel/arch_x86_64/memory_bandwidth/C/pthread/SSE2-Flush/single-cas-32-fail/PARAMETERS" );
      bi_put( "BENCHIT_PROGRESS_DIR", "progress" );
      bi_put( "BENCHIT_RUN_ACCURACY", "2" );
      bi_put( "BENCHIT_RUN_COREDUMPLIMIT", "0" );
      bi_put( "BENCHIT_RUN_EMAIL_ADDRESS", "" );
      bi_put( "BENCHIT_RUN_LINEAR", "0" );
      bi_put( "BENCHIT_RUN_MAX_MEMORY", "0" );
      bi_put( "BENCHIT_RUN_OUTPUT_DIR", "/home/herm/BenchITv6/output" );
      bi_put( "BENCHIT_RUN_QUEUENAME", "" );
      bi_put( "BENCHIT_RUN_REDIRECT_CONSOLE", "" );
      bi_put( "BENCHIT_RUN_TEST", "0" );
      bi_put( "BENCHIT_RUN_TIMELIMIT", "3600" );
      bi_put( "BENCHIT_USE_VAMPIR_TRACE", "0" );
      bi_put( "BR", "7" );
      bi_put( "CLUTTER_IM_MODULE", "xim" );
      bi_put( "COLORTERM", "gnome-terminal" );
      bi_put( "COMMENT", "" );
      bi_put( "COMPIZ_CONFIG_PROFILE", "ubuntu" );
      bi_put( "CONFIGURE_MODE", "COMPILE" );
      bi_put( "CURDIR", "/home/herm/BenchITv6/kernel/arch_x86_64/memory_bandwidth/C/pthread/SSE2-Flush/single-cas-32-fail" );
      bi_put( "DBUS_SESSION_BUS_ADDRESS", "unix:abstract=/tmp/dbus-JwVGpkfXPP" );
      bi_put( "DEFAULTS_PATH", "/usr/share/gconf/ubuntu.default.path" );
      bi_put( "DESKTOP_SESSION", "ubuntu" );
      bi_put( "DISPLAY", ":0.0" );
      bi_put( "GDMSESSION", "ubuntu" );
      bi_put( "GDM_LANG", "en_US" );
      bi_put( "GNOME_DESKTOP_SESSION_ID", "this-is-deprecated" );
      bi_put( "GNOME_KEYRING_CONTROL", "/run/user/1000/keyring-u178We" );
      bi_put( "GNOME_KEYRING_PID", "1543" );
      bi_put( "GPG_AGENT_INFO", "/run/user/1000/keyring-u178We/gpg:0:1" );
      bi_put( "GTK_IM_MODULE", "ibus" );
      bi_put( "GTK_MODULES", "overlay-scrollbar:unity-gtk-module" );
      bi_put( "HLL", "C" );
      bi_put( "HOME", "/home/herm" );
      bi_put( "IFS", "' 	" );
      bi_put( "IM_CONFIG_PHASE", "1" );
      bi_put( "INSTANCE", "" );
      bi_put( "JOB", "dbus" );
      bi_put( "KERNELBASEDIR", "/home/herm/BenchITv6/kernel" );
      bi_put( "KERNELDIR", "/home/herm/BenchITv6/kernel/arch_x86_64/memory_bandwidth/C/pthread/SSE2-Flush/single-cas-32-fail" );
      bi_put( "KERNELNAME_FULL", "" );
      bi_put( "LANGUAGE", "en_US" );
      bi_put( "LC_ADDRESS", "de_CH.UTF-8" );
      bi_put( "LC_IDENTIFICATION", "de_CH.UTF-8" );
      bi_put( "LC_MEASUREMENT", "de_CH.UTF-8" );
      bi_put( "LC_MONETARY", "de_CH.UTF-8" );
      bi_put( "LC_NAME", "de_CH.UTF-8" );
      bi_put( "LC_NUMERIC", "de_CH.UTF-8" );
      bi_put( "LC_PAPER", "de_CH.UTF-8" );
      bi_put( "LC_TELEPHONE", "de_CH.UTF-8" );
      bi_put( "LC_TIME", "de_CH.UTF-8" );
      bi_put( "LESSCLOSE", "/usr/bin/lesspipe %s %s" );
      bi_put( "LESSOPEN", "| /usr/bin/lesspipe %s" );
      bi_put( "LOCAL_BENCHITC_COMPILER", "gcc  -O2  -DDEBUGLEVEL=0" );
      bi_put( "LOCAL_KERNEL_COMPILER", "gcc" );
      bi_put( "LOCAL_KERNEL_COMPILERFLAGS", " -O0 -I. -I/home/herm/BenchITv6 -I/home/herm/BenchITv6/tools -I/home/herm/BenchITv6/tools/hw_detect" );
      bi_put( "LOCAL_LINKERFLAGS", "-lm " );
      bi_put( "LOGNAME", "herm" );
      bi_put( "LS_COLORS", "rs=0:di=01;34:ln=01;36:mh=00:pi=40;33:so=01;35:do=01;35:bd=40;33;01:cd=40;33;01:or=40;31;01:su=37;41:sg=30;43:ca=30;41:tw=30;42:ow=34;42:st=37;44:ex=01;32:*.tar=01;31:*.tgz=01;31:*.arj=01;31:*.taz=01;31:*.lzh=01;31:*.lzma=01;31:*.tlz=01;31:*.txz=01;31:*.zip=01;31:*.z=01;31:*.Z=01;31:*.dz=01;31:*.gz=01;31:*.lz=01;31:*.xz=01;31:*.bz2=01;31:*.bz=01;31:*.tbz=01;31:*.tbz2=01;31:*.tz=01;31:*.deb=01;31:*.rpm=01;31:*.jar=01;31:*.war=01;31:*.ear=01;31:*.sar=01;31:*.rar=01;31:*.ace=01;31:*.zoo=01;31:*.cpio=01;31:*.7z=01;31:*.rz=01;31:*.jpg=01;35:*.jpeg=01;35:*.gif=01;35:*.bmp=01;35:*.pbm=01;35:*.pgm=01;35:*.ppm=01;35:*.tga=01;35:*.xbm=01;35:*.xpm=01;35:*.tif=01;35:*.tiff=01;35:*.png=01;35:*.svg=01;35:*.svgz=01;35:*.mng=01;35:*.pcx=01;35:*.mov=01;35:*.mpg=01;35:*.mpeg=01;35:*.m2v=01;35:*.mkv=01;35:*.webm=01;35:*.ogm=01;35:*.mp4=01;35:*.m4v=01;35:*.mp4v=01;35:*.vob=01;35:*.qt=01;35:*.nuv=01;35:*.wmv=01;35:*.asf=01;35:*.rm=01;35:*.rmvb=01;35:*.flc=01;35:*.avi=01;35:*.fli=01;35:*.flv=01;35:*.gl=01;35:*.dl=01;35:*.xcf=01;35:*.xwd=01;35:*.yuv=01;35:*.cgm=01;35:*.emf=01;35:*.axv=01;35:*.anx=01;35:*.ogv=01;35:*.ogx=01;35:*.aac=00;36:*.au=00;36:*.flac=00;36:*.mid=00;36:*.midi=00;36:*.mka=00;36:*.mp3=00;36:*.mpc=00;36:*.ogg=00;36:*.ra=00;36:*.wav=00;36:*.axa=00;36:*.oga=00;36:*.spx=00;36:*.xspf=00;36:" );
      bi_put( "MANDATORY_PATH", "/usr/share/gconf/ubuntu.mandatory.path" );
      bi_put( "OLDCWD", "/home/herm/BenchITv6/kernel/arch_x86_64/memory_bandwidth/C/pthread/SSE2-Flush/single-cas-32-fail" );
      bi_put( "OLDIR", "/home/herm/BenchITv6/kernel/arch_x86_64/memory_bandwidth/C/pthread/SSE2-Flush/single-cas-32-fail" );
      bi_put( "OLDPWD", "/home/herm/BenchITv6/kernel/arch_x86_64/memory_bandwidth/C/pthread/SSE2-Flush/single-cas-32-fail" );
      bi_put( "OMP_DYNAMIC", "FALSE" );
      bi_put( "OMP_NESTED", "FALSE" );
      bi_put( "OMP_NUM_THREADS", "1" );
      bi_put( "OPTIND", "1" );
      bi_put( "PATH", "/home/herm/BenchITv6/tools:/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin:/usr/games:/usr/local/games" );
      bi_put( "PPID", "9333" );
      bi_put( "PS1", "$ " );
      bi_put( "PS2", "> " );
      bi_put( "PS4", "+ " );
      bi_put( "PWD", "/home/herm/BenchITv6/tools" );
      bi_put( "QT4_IM_MODULE", "ibus" );
      bi_put( "QT_IM_MODULE", "ibus" );
      bi_put( "QT_QPA_PLATFORMTHEME", "appmenu-qt5" );
      bi_put( "SCRIPTNAME", "COMPILE.SH" );
      bi_put( "SELINUX_INIT", "YES" );
      bi_put( "SESSION", "ubuntu" );
      bi_put( "SESSIONTYPE", "gnome-session" );
      bi_put( "SESSION_MANAGER", "local/herm-ThinkPad-X220:@/tmp/.ICE-unix/1687,unix/herm-ThinkPad-X220:/tmp/.ICE-unix/1687" );
      bi_put( "SHELL", "/bin/bash" );
      bi_put( "SHELLSCRIPT_DEBUG", "0" );
      bi_put( "SHLVL", "1" );
      bi_put( "SSH_AUTH_SOCK", "/run/user/1000/keyring-u178We/ssh" );
      bi_put( "TERM", "xterm" );
      bi_put( "TEXTDOMAIN", "im-config" );
      bi_put( "TEXTDOMAINDIR", "/usr/share/locale/" );
      bi_put( "UPSTART_SESSION", "unix:abstract=/com/ubuntu/upstart-session/1000/1545" );
      bi_put( "USER", "herm" );
      bi_put( "VTE_VERSION", "3409" );
      bi_put( "WINDOWID", "73400332" );
      bi_put( "XAUTHORITY", "/home/herm/.Xauthority" );
      bi_put( "XDG_CONFIG_DIRS", "/etc/xdg/xdg-ubuntu:/usr/share/upstart/xdg:/etc/xdg" );
      bi_put( "XDG_CURRENT_DESKTOP", "Unity" );
      bi_put( "XDG_DATA_DIRS", "/usr/share/ubuntu:/usr/share/gnome:/usr/local/share/:/usr/share/" );
      bi_put( "XDG_GREETER_DATA_DIR", "/var/lib/lightdm-data/herm" );
      bi_put( "XDG_MENU_PREFIX", "gnome-" );
      bi_put( "XDG_RUNTIME_DIR", "/run/user/1000" );
      bi_put( "XDG_SEAT", "seat0" );
      bi_put( "XDG_SEAT_PATH", "/org/freedesktop/DisplayManager/Seat0" );
      bi_put( "XDG_SESSION_ID", "c2" );
      bi_put( "XDG_SESSION_PATH", "/org/freedesktop/DisplayManager/Session0" );
      bi_put( "XDG_VTNR", "7" );
      bi_put( "XMODIFIERS", "@im=ibus" );
      bi_put( "_", "/bin/sh" );
      bi_put( "_CMDLINE_VARLIST", "BENCHIT_KERNELBINARY BENCHIT_KERNELBINARY_ARGS BENCHIT_CMDLINE_ARG_FILENAME_COMMENT BENCHIT_CMDLINE_ARG_PARAMETER_FILE BENCHIT_CMDLINE_ARG_IGNORE_PARAMETER_FILE BENCHIT_NODENAME BENCHIT_CROSSCOMPILE BENCHIT_CMDLINE_ARG_NUM_CPUS BENCHIT_CMDLINE_ARG_NUM_PROCESSES BENCHIT_CMDLINE_ARG_NUM_THREADS_PER_PROCESS BENCHIT_CMDLINE_ARG_RUN_CLEAN BENCHIT_CMDLINE_ARG_RUN_COREDUMPLIMIT BENCHIT_CMDLINE_ARG_RUN_EMAIL_ADDRESS BENCHIT_CMDLINE_ARG_RUN_MAX_MEMORY BENCHIT_CMDLINE_ARG_RUN_QUEUENAME BENCHIT_CMDLINE_ARG_RUN_QUEUETIMELIMIT BENCHIT_CMDLINE_ARG_RUN_REDIRECT_CONSOLE BENCHIT_CMDLINE_ARG_RUN_TEST BENCHIT_CMDLINE_ARG_RUN_USE_MPI BENCHIT_CMDLINE_ARG_RUN_USE_OPENMP " );
      bi_put( "_VARLIST", "'BENCHITROOT" );
      bi_put( "myfile", "tools/stringlib.h" );
      bi_put( "myval", "" );
      bi_put( "myvar", "BENCHIT_CMDLINE_ARG_RUN_USE_OPENMP" );
      bi_put( "BENCHIT_KERNEL_FILE_VERSION_BENCHSCRIPT_C", "NO REVISION (Fri Sep 11 14:26:20 2009)" );
      bi_put( "BENCHIT_KERNEL_FILE_VERSION_FILEVERSION", "NO REVISION (Fri May  6 15:50:25 2016)" );
      bi_put( "BENCHIT_KERNEL_FILE_VERSION_ALIGNED_MEMORY_H", "NO REVISION (Fri Sep 11 14:26:20 2009)" );
      bi_put( "BENCHIT_KERNEL_FILE_VERSION_BIENVHASH_TEMPLATE_C", "NO REVISION (Fri Sep 11 14:26:20 2009)" );
      bi_put( "BENCHIT_KERNEL_FILE_VERSION_LOC_REPL", "NO REVISION (Fri Sep 11 14:26:20 2009)" );
      bi_put( "BENCHIT_KERNEL_FILE_VERSION_CMDLINEPARAMS", "NO REVISION (Fri Sep 11 14:26:20 2009)" );
      bi_put( "BENCHIT_KERNEL_FILE_VERSION_BMERGE_SH", "NO REVISION (Fri Sep 11 14:26:20 2009)" );
      bi_put( "BENCHIT_KERNEL_FILE_VERSION_STRINGLIB_H", "NO REVISION (Fri Sep 11 14:26:20 2009)" );
      bi_put( "BENCHIT_KERNEL_FILE_VERSION_CONFIGURE", "NO REVISION (Fri Sep 11 14:26:20 2009)" );
      bi_put( "BENCHIT_KERNEL_FILE_VERSION_ENVHASHBUILDER_C", "NO REVISION (Fri Sep 11 14:26:20 2009)" );
      bi_put( "BENCHIT_KERNEL_FILE_VERSION_CBLAS", "NO REVISION, UNABLE TO READ (Wed Sep 24 22:21:12 2014)" );
      bi_put( "BENCHIT_KERNEL_FILE_VERSION_FIRSTTIME", "NO REVISION (Mon Sep 14 13:36:18 2009)" );
      bi_put( "BENCHIT_KERNEL_FILE_VERSION_ERROR_H", "NO REVISION (Fri Sep 11 14:26:20 2009)" );
      bi_put( "BENCHIT_KERNEL_FILE_VERSION_BENCHSCRIPT_H", "NO REVISION (Fri Sep 11 14:26:20 2009)" );
      bi_put( "BENCHIT_KERNEL_FILE_VERSION_BIENVHASH_H", "NO REVISION (Fri Sep 11 14:26:20 2009)" );
      bi_put( "BENCHIT_KERNEL_FILE_VERSION_TMP_ENV", "NO REVISION (Fri May  6 15:50:25 2016)" );
      bi_put( "BENCHIT_KERNEL_FILE_VERSION_BIENVHASH_TEMPLATE_JAVA", "NO REVISION (Fri Sep 11 14:26:20 2009)" );
      bi_put( "BENCHIT_KERNEL_FILE_VERSION_BLAS", "NO REVISION, UNABLE TO READ (Wed Sep 24 22:20:01 2014)" );
      bi_put( "BENCHIT_KERNEL_FILE_VERSION_ENVIRONMENTS", "NO REVISION, UNABLE TO READ (Wed Sep 24 22:23:35 2014)" );
      bi_put( "BENCHIT_KERNEL_FILE_VERSION_LOC_CONVERT_SH", "NO REVISION (Fri Sep 11 14:26:20 2009)" );
      bi_put( "BENCHIT_KERNEL_FILE_VERSION_FILEVERSION_C", "*/ (Fri Sep 11 14:26:20 2009)" );
      bi_put( "BENCHIT_KERNEL_FILE_VERSION_FEATURES", "NO REVISION (Mon Sep 14 13:36:18 2009)" );
      bi_put( "BENCHIT_KERNEL_FILE_VERSION_HELPER_SH", "1.14 (Fri Sep 11 14:26:20 2009)" );
      bi_put( "BENCHIT_KERNEL_FILE_VERSION_QUICKVIEW_SH", "NO REVISION (Fri Sep 11 14:26:20 2009)" );
      bi_put( "BENCHIT_KERNEL_FILE_VERSION_REFERENCE_RUN_SH", "NO REVISION (Fri Sep 11 14:26:20 2009)" );
      bi_put( "BENCHIT_KERNEL_FILE_VERSION_COMPILERVERSION", "NO REVISION, UNABLE TO READ (Wed Sep 24 22:33:31 2014)" );
      bi_put( "BENCHIT_KERNEL_FILE_VERSION_CHANGE_SH_SH", "NO REVISION (Fri Sep 11 14:26:20 2009)" );
      bi_put( "BENCHIT_KERNEL_FILE_VERSION_STRINGLIB_C", "NO REVISION (Fri Sep 11 14:26:20 2009)" );
      bi_put( "BENCHIT_KERNEL_FILE_VERSION_HW_DETECT", "NO REVISION, UNABLE TO READ (Wed Sep 24 22:23:51 2014)" );
      bi_put( "BENCHIT_KERNEL_FILE_VERSION_ENVHASHBUILDER", "NO REVISION (Fri May  6 15:50:25 2016)" );
   }
}
